//		MPI code in C++.
//		See [Gropp/Lusk/Skjellum, "Using MPI", p.33/41 etc.]
//		and  /opt/mpich/include/mpi2c++/comm.h  for details

#include <iostream>
#include <cmath>
using namespace std;
#include "geom.h"
#include "getmatrix.h"
#include "jacsolve.h"
#include "userset.h"

int main(int , char** )
{
    const int numprocs = 1;
    const int myrank   = 0;

    if (myrank == 0)
    { cout << "\n There are " << numprocs << " processes running.\n \n"; }

    const int procx = static_cast<int>(sqrt(numprocs+0.0));
    const int procy = procx;

    if (procy * procx != numprocs)
    {
      cout << "\n Wrong number of processors !\n \n";
    }
    else
    {
      int neigh[4], color;
      IniGeom(myrank, procx, procy, neigh, color); // Topology of domain decomposition

      double xl,xr,yb,yt;                          // subdomain coordinates
      IniCoord(myrank, procx, procy, xl, xr, yb, yt);

      //bool ScaleUp = !true;
      int nx,ny,NXglob,NYglob;  /* number of local intervals on (xl,xr)=:nx, (yb,yt)=:ny */
	nx = 1000;
	ny = 1000;
	NXglob = nx*procx;
	NYglob = ny*procy;
    cout << "Intervalls: " << NXglob << " x " << NYglob << endl;

// Initialze the mesh
      int nnode, nelem;        // number of ccordinates, number of triangular elements
      double *xc;              // coordinates         xc[nnode,2]
      int    *ia;              // element conectivity ia[nelem,3]
      GetMesh(nx, ny, xl, xr, yb, yt, nnode, xc, nelem, ia);
      //DebugMesh(nnode, xc, nelem, 3, ia);  {int ijk; cin >> ijk;}

// Matrix Graph
      int nnz(-1);                           // number of non-zero eelemnts in matrix
      double *sk;                            // sk[nnz]  non-zero entries
      int    *ik,                            // ik[nnz]  column indices
             *id;                            // id[nnode+1] index of row begin
      Get_Matrix_Pattern(nelem, 3, ia, nnz, id, ik, sk);
      //DebugMatrix(nnode, id, ik, sk);  {int ijk; cin >> ijk;}

      /* allocate memory */
      double *u = new double [nnode],        /* temperature       */
             *f = new double [nnode];        /* r.h.s.            */

      /* matrix, rhs generation including Dirichlet B.C. */

      GetMatrix (nelem, 3, ia, nnode, xc, nnz, id, ik, sk, f);
      //DebugMatrix(nnode, id, ik, sk);  {int ijk; cin >> ijk;}

      SetU(nx, ny, u);                       /* set solution/boundary values */
      ApplyDirichletBC(nx, ny, neigh, u, id, ik, sk, f); /* apply B.C.       */
      //DebugMatrix(nnode, id, ik, sk);

      double tstart = clock();                       // timing

      JacobiSolve(nnode, id, ik, sk, f, u );

      double t1 = (clock() - tstart)/CLOCKS_PER_SEC;                 // timing
      cout << "JacobiSolve: timing in sec. : " << t1 << endl;


      SaveVectorP(myrank, "t.dat", u, nx, ny, xl, xr, yb, yt);

      delete [] ia; delete [] xc;
      delete [] id; delete [] ik; delete [] sk; delete [] f; delete [] u;
  }
  return 0;
}


